#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from zope.interface import Interface
from zope.schema import Bool, Choice, Set

from pyams_content.interfaces import GUEST_ROLE, IBaseContent, MANAGER_ROLE
from pyams_content.shared.common.interfaces import CONTENT_TYPES_VOCABULARY
from pyams_content.shared.common.interfaces.types import ALL_DATA_TYPES_VOCABULARY
from pyams_content.shared.site.interfaces import IBaseSiteItem, ISiteElement
from pyams_content.shared.view import IWfView
from pyams_content.shared.view.interfaces import RELEVANCE_ORDER, USER_VIEW_ORDER_VOCABULARY
from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import DESIGNER_ROLE
from pyams_security.schema import PrincipalsSet
from pyams_sequence.interfaces import ISequentialIdTarget, IInternalReference
from pyams_sequence.schema import InternalReferenceField

from pyams_content import _


class ISearchManagerInfo(IInternalReference):
    """Search manager interface"""

    reference = InternalReferenceField(title=_("Main search engine"),
                                       description=_("Search folder handling main site search. You can search a "
                                                     "reference using '+' followed by internal number, of by "
                                                     "entering text matching content title."),
                                       required=False)

    name = I18nTextLineField(title=_("Search engine name"),
                             description=_("Name given to the search engine"),
                             required=False)

    description = I18nTextLineField(title=_("Description"),
                                    description=_("Description given to the search engine"),
                                    required=False)


class ISearchFolderRoles(Interface):
    """Search folder roles"""

    managers = PrincipalsSet(title=_("Managers"),
                             description=_("Managers can handle main operations in tool's workflow, like publish "
                                           "or retire contents"),
                             role_id=MANAGER_ROLE,
                             required=False)

    designers = PrincipalsSet(title=_("Designers"),
                              description=_("Designers are users which are allowed to manage presentation templates"),
                              role_id=DESIGNER_ROLE,
                              required=False)

    guests = PrincipalsSet(title=_("Guests"),
                           description=_("Guests are users which are allowed to view contents with restricted access"),
                           role_id=GUEST_ROLE,
                           required=False)


class ISearchFolder(IBaseContent, IBaseSiteItem, ISiteElement, IWfView, ISequentialIdTarget):
    """Search folder interface"""

    order_by = Choice(title=_("Order by"),
                      description=_("Property to use to sort results; publication date can be different from first "
                                    "publication date for contents which have been retired and re-published with a "
                                    "different publication date"),
                      vocabulary=USER_VIEW_ORDER_VOCABULARY,
                      required=False,
                      default=RELEVANCE_ORDER)

    visible_in_list = Bool(title=_("Visible in folders list"),
                           description=_("If 'no', folder will not be displayed into folders list"),
                           required=True,
                           default=True)

    navigation_title = I18nTextLineField(title=_("Navigation title"),
                                         description=_("Folder's title displayed in navigation pages; "
                                                       "original title will be used if none is specified"),
                                         required=False)

    selected_content_types = Set(title=_("Selected content types"),
                                 description=_("Searched content types; leave empty for all"),
                                 value_type=Choice(vocabulary=CONTENT_TYPES_VOCABULARY),
                                 required=False)

    selected_datatypes = Set(title=_("Selected data types"),
                             description=_("Searched data types; leave empty for all"),
                             value_type=Choice(vocabulary=ALL_DATA_TYPES_VOCABULARY),
                             required=False)
